package com.bizmatics.controller.web;


import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.model.AlarmPower;
import com.bizmatics.service.AlarmPowerService;
import com.bizmatics.service.vo.ApCountVO;
import com.bizmatics.service.vo.CommonIcoVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 电力告警
 *
 * @author ya
 * @since 2021-07-07
 */
@RestController
@RequestMapping("/alarmPower")
public class AlarmPowerController {

    @Autowired
    private AlarmPowerService alarmPowerService;


    /**
     *
     * 查询电力告警总数
     * @param siteId 站点id
     * @return
     */
    @GetMapping("/count")
    public ApiResult<ApCountVO> count(@RequestParam(required = false) Integer siteId,
                                      @RequestParam(required = false) Date startTime,
                                      @RequestParam(required = false) Date endTime) {
        return ApiResult.success(alarmPowerService.selectCount(siteId,startTime,endTime));
    }


    /**
     * 告警趋势图
     * @param startTime 天数
     * @param endTime 结束时间
     * @param siteId 站点id
     * @return
     */
    @GetMapping("/trendIco")
    public ApiResult<List<CommonIcoVO>> selectByDate(@RequestParam Date startTime,
                                                     @RequestParam Date endTime,
                                                     @RequestParam(required = false) Integer siteId){
        return ApiResult.success(alarmPowerService.selectByDate(startTime,endTime,siteId));
    }


    /**
     * 分页
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param status 状态
     * @param siteId 站点
     * @param current 页数
     * @param size 条数
     * @return
     */
    @GetMapping("/page")
    public ApiResult<CommonPage<AlarmPower>> page(@RequestParam(required = false) Date startTime,
                                                  @RequestParam(required = false) Date endTime,
                                                  @RequestParam(required = false) Integer status,
                                                  @RequestParam(required = false) Integer siteId,
                                                  @RequestParam Integer current,
                                                  @RequestParam Integer size){
        return ApiResult.success(alarmPowerService.page(startTime, endTime, status, siteId, current, size));
    }

    @RequestMapping("LoopStatusList")
    public ApiResult<List<Map<String,Object>>> getLoopStatus(@RequestParam Integer siteId){
        return ApiResult.success(alarmPowerService.getLoopStatusList(siteId));
    }


    /**
     * 导出
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param status 状态
     * @param siteId 站点
     * @return
     */
    @GetMapping("/export")
    public ApiResult<String> export(@RequestParam(required = false) Date startTime,
                                                  @RequestParam(required = false) Date endTime,
                                                  @RequestParam(required = false) Integer status,
                                                  @RequestParam(required = false) Integer siteId){
        return ApiResult.success(alarmPowerService.export(startTime, endTime, status, siteId));
    }


    /**
     * 运行状态
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param siteId 站点
     * @return
     */
    @GetMapping("/runStatus")
    public ApiResult<List<CommonIcoVO>> runStatus(@RequestParam Date startTime,
                                                  @RequestParam Date endTime,
                                                  @RequestParam Integer siteId){
        return ApiResult.success(alarmPowerService.runStatus(startTime,endTime,siteId));
    }
}

